@@ -1,18 +1,20 @@
 import re
-from .engine import Engine
-from .support import DBError
-from typing import Collection
+from executor import DBError
 from functools import lru_cache
+from .log_support import sql_log
+from .dialect import Dialect, require_limit
+from typing import Collection, Union, List, Tuple
 from .constant import CACHE_SIZE, NAMED_REGEX, LIMIT_1
 
 
 def insert_sql(table: str, cols: Collection[str]):
     cols = cols if isinstance(cols, tuple) else tuple(cols)
-    return Engine.create_insert_sql_intf(table, cols)
+    return Dialect.create_insert_sql(table, cols)
+
 
 def insert_sql_args(table: str, **kwargs):
     cols, args = zip(*kwargs.items())
-    sql = Engine.create_insert_sql_intf(table, cols)
+    sql = Dialect.create_insert_sql(table, cols)
     return sql, args
 
 
@@ -24,7 +26,7 @@ def batch_insert_sql_args(table: str, *args):
     args = get_batch_args(*args)
     args = [zip(*arg.items()) for arg in args]  # [(cols, args)]
     cols, args = zip(*args)  # (cols), (args)
-    sql = Engine.create_insert_sql_intf(table, cols[0])
+    sql = Dialect.create_insert_sql(table, cols[0])
     return sql, args
 
 
@@ -55,16 +57,6 @@ def limit_one_sql_args(sql: str, *args):
     return sql, args
 
 
-def require_limit(sql: str):
-    lower_sql = sql.lower()
-    if 'limit' not in lower_sql:
-        return True
-    idx = lower_sql.rindex('limit')
-    if idx > 0 and ')' in lower_sql[idx:]:
-        return True
-    return False
-
-
 def is_mapping(sql: str):
     return ':' in sql
 
@@ -82,3 +74,22 @@ def get_mapping_sql_args(sql: str, *args, **kwargs):
         raise DBError("Placeholder sql expect '*args' should not be empty.")
 
     return sql, args
+
+
+def try_mapping(function, sql, *args, **kwargs):
+    sql_log(function, sql, *args, **kwargs)
+    return get_mapping_sql_args(sql, *args, **kwargs)
+
+
+def get_table_select_sql(table_name: str, where: str, limit: Union[int, Tuple[int], List[int]], *columns):
+    columns = ','.join([col if '(' in col else '{}'.format(col) for col in columns]) if columns else Dialect.get_table_columns(table_name)
+
+    if limit:
+        if isinstance(limit, int):
+            return 'SELECT {} FROM {} {} LIMIT ?'.format(columns, table_name, where)
+        elif (isinstance(limit, Tuple) or isinstance(limit, List)) and len(limit) == 2:
+            return 'SELECT {} FROM {} {} LIMIT ? OFFSET ?'.format(columns, table_name, where)
+        else:
+            raise ValueError("The type of the parameter 'limit' must be 'int' or tuple, list, and it length is 2.")
+    else:
+        return 'SELECT {} FROM {} {}'.format(columns, table_name, where)
